create or replace
PACKAGE report_util
AS
  /* Global variables */
  empty_vc_arr APEX_APPLICATION_GLOBAL.vc_arr2;
--------------------------------------------------------------------------------
  /* Procedures and functions */
  PROCEDURE ajax_report(
    pName       IN VARCHAR2,
    pStartRow   IN VARCHAR2 DEFAULT NULL,
    pMaxRows    IN VARCHAR2 DEFAULT NULL,
    pSortCol    IN VARCHAR2 DEFAULT NULL,
    pSeqId      IN NUMBER DEFAULT 1,
    pItems      IN APEX_APPLICATION_GLOBAL.vc_arr2 DEFAULT empty_vc_arr,
    pValues     IN APEX_APPLICATION_GLOBAL.vc_arr2 DEFAULT empty_vc_arr
  );
--------------------------------------------------------------------------------
  PROCEDURE ajax_report_js;
--------------------------------------------------------------------------------
  PROCEDURE p(
    p_html IN OUT NOCOPY CLOB,
    p_append IN VARCHAR
  );
--------------------------------------------------------------------------------
  PROCEDURE print_html(
    p_html IN CLOB
  );
--------------------------------------------------------------------------------
END;
/
create or replace
PACKAGE BODY                                     report_util
AS
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE ajax_report(
    pName       IN VARCHAR2,
    pStartRow   IN VARCHAR2 DEFAULT NULL,
    pMaxRows    IN VARCHAR2 DEFAULT NULL,
    pSortCol    IN VARCHAR2 DEFAULT NULL,
    pSeqId      IN NUMBER DEFAULT 1,
    pItems      IN APEX_APPLICATION_GLOBAL.vc_arr2 DEFAULT empty_vc_arr,
    pValues     IN APEX_APPLICATION_GLOBAL.vc_arr2 DEFAULT empty_vc_arr
  )
  AS

    l_desc_t    dbms_sql.desc_tab2;
    l_desc_c    dbms_sql.desc_tab;
    
    l_status      NUMBER;
    l_col_cnt     INTEGER;
    l_row_cnt     INTEGER;
    
    l_cursor      INTEGER;
    l_start_row   INTEGER;
    l_max_rows    INTEGER;
    l_last_row    INTEGER;
    l_next_start  INTEGER;
    l_total_rows  INTEGER;

    l_sql         VARCHAR2(32700);
    l_cvalue      VARCHAR2(32700);
    l_table       VARCHAR2(32700);
    l_th          VARCHAR2(32700);
    l_td          VARCHAR2(32700);
    
    l_sort_col    VARCHAR2(255);
    l_sort_ord    VARCHAR2(255);
    
    l_drop_id     VARCHAR2(255);
    l_asc_img     VARCHAR2(255);
    l_desc_img    VARCHAR2(255);
    l_sort_img    VARCHAR2(255);
    l_prev_img    VARCHAR2(255);
    l_next_img    VARCHAR2(255);
    
    l_html        CLOB;
    
  BEGIN
    --
    /* HTML tag templates */
    l_table := '<table cellpadding="0" border="0" cellspacing="0" summary="" class="report-standard">';
    l_th    := '<th class="header"><div class="rpt-sort">';
    l_td    := '<td class="data">';
    --
    /* Get session state */
    SELECT coalesce(pStartRow, c001, '1')                               AS start_row,
      coalesce(pMaxRows, c002, '15')                                    AS max_rows,
      coalesce(pSortCol, c003, '1')                                     AS sort_column,
      CASE WHEN pSortCol = c003 THEN
        DECODE(c004,'ASC','DESC','DESC','ASC','ASC')
      WHEN pSortCol != c003 THEN
        'ASC'
      ELSE
       coalesce(c004,'ASC')
      END                                                               AS sort_order,
      coalesce(c011, apex_application.g_image_prefix || 'ws/ddown.gif') AS sort_asc,
      coalesce(c012, apex_application.g_image_prefix || 'ws/dup.gif')   AS sort_desc,
      coalesce(c013, apex_application.g_image_prefix || 'jtfupree.gif') AS pagin_prev,
      coalesce(c014, apex_application.g_image_prefix || 'jtfunexe.gif') AS pagin_next,
      clob001                                                           AS sql_query
    INTO l_start_row,
      l_max_rows,
      l_sort_col,
      l_sort_ord,
      l_asc_img,
      l_desc_img,
      l_prev_img,
      l_next_img,
      l_sql
    FROM apex_collections
    WHERE collection_name = pName
    AND seq_id = pSeqId
    ;
    --
    /* End row */
    l_last_row := l_start_row + l_max_rows -1;
    --
    /* Sort img */
    IF l_sort_ord = 'DESC' THEN
      l_sort_img := l_desc_img;
    ELSE
      l_sort_img := l_asc_img;
    END IF;
    --
    /* DIV tag id */
    l_drop_id := 'htmldb_' || pName || pSeqId || '_catch';
    --
    /* Report query */
    l_sql := '
      SELECT * FROM(
        SELECT a.*, row_number() OVER (ORDER BY ' 
        || l_sort_col 
        || ' ' 
        || l_sort_ord 
        || ') rn, COUNT(1) over() mrn
        FROM(' || l_sql || ') a
      ) WHERE rn BETWEEN ' || l_start_row || ' AND ' || l_last_row
    ;
    --
    --
    /* Print report surrounding tags */
    p(l_html,'<div><div id="' || l_drop_id || '">');
    --
    p(l_html,'<table cellpadding="0" border="0" cellspacing="0" summary="">');
    p(l_html,'<tr><td>');
    --
    /* Set cursor */
    l_cursor := dbms_sql.open_cursor;
    --
    dbms_sql.parse ( l_cursor, l_sql, dbms_sql.native );
    --
    /* get bind variables from session state */
    wwv_flow_utilities.perform_binds( l_cursor, l_sql );
    --
    dbms_sql.describe_columns( l_cursor, l_col_cnt, l_desc_c );
    --
    --
    /* Report start */
    p(l_html,l_table);
    --
    /* Print headings */
    p(l_html,'<thead>');
    p(l_html,'<tr>');
    --
    FOR i IN 1 .. l_col_cnt - 2
    LOOP
    --
      p(l_html,l_th 
        || '<a href="javascript:$.htmldbAjaxReport({'
        || 'pSeqId:' || pSeqId || ','
        || 'pName:''' || pName || ''','
        --|| 'pStartRow:1,'
        || 'pMaxRows:' || l_max_rows || ','
        || 'pSortCol:''' || trim(l_desc_c(i).col_name) || ''''
        || '})">' 
        || trim(l_desc_c(i).col_name)
      );
      /* Add image to heading if report is sorted by this column */
      IF trim(l_desc_c(i).col_name) = l_sort_col THEN
        p(l_html,'<img alt="" style="vertical-align:middle" src="' || l_sort_img || '" />');
      END IF;
      p(l_html,'</a></div></th>');
    --
    END LOOP;
    --
    /* End of headers. Start body */
    p(l_html,'</tr></thead><tbody>');
    --
    /* Report data */
    --
    dbms_sql.describe_columns2( l_cursor, l_col_cnt, l_desc_t );
    --
    FOR i IN 1 .. l_col_cnt
    LOOP
      dbms_sql.define_column( l_cursor, i, l_cvalue, 32765);
    END LOOP;
    --
    l_status := dbms_sql.execute( l_cursor );
    --
    l_row_cnt := 0;
    --
    LOOP
      EXIT
    WHEN l_row_cnt >= l_max_rows OR dbms_sql.fetch_rows( l_cursor ) <= 0;
      --
      l_row_cnt := l_row_cnt + 1;
      --
      p(l_html,'<tr>');
      FOR i IN 1 .. l_col_cnt - 2
      LOOP
        dbms_sql.column_value( l_cursor, i, l_cvalue );
        p(l_html,l_td || trim(l_cvalue) || '</td>');
      END LOOP;
      p(l_html,'</tr>');
    END LOOP;
    --
    /* Get last printed data row number */
    dbms_sql.column_value( l_cursor, l_col_cnt - 1, l_cvalue );
    l_last_row := trim(l_cvalue);
    --
    /* Get report total rows */
    dbms_sql.column_value( l_cursor, l_col_cnt, l_cvalue );
    l_total_rows := trim(l_cvalue);    
    --
    /* Close cursor */
    IF dbms_sql.is_open(l_cursor) THEN
      dbms_sql.close_cursor(l_cursor);
    END IF;
    --
    /* End of report data */
    p(l_html,'</tbody>');
    p(l_html,'</table>');
    p(l_html,'</td></tr>');
    --
    /* Pagination if report have data */
    p(l_html,'<tr><td>');
    IF l_row_cnt > 0 THEN
      p(l_html,'<div style="text-align:right;padding:10px 3px">');
      IF l_start_row > 1 THEN
        l_next_start := l_start_row - l_max_rows;
        p(l_html,'<a href="javascript:$.htmldbAjaxReport({'
          || 'pSeqId:' || pSeqId || ','
          || 'pName:''' || pName || ''','
          || 'pMaxRows:' || l_max_rows || ','
          || 'pStartRow:' || l_next_start
          || '})">'
        );
        p(l_html,'<img style="vertical-align:top" src="' || l_prev_img || '" alt="" />');
        p(l_html,'</a>&nbsp;');
      END IF;
      p(l_html,'<span class="fielddata">Rows ');
      p(l_html,l_start_row);
      p(l_html,' - ');
      p(l_html,l_last_row);
      p(l_html,' of ');
      p(l_html,l_total_rows);
      p(l_html,'</span>');
      IF l_last_row < l_total_rows THEN
        l_next_start := l_last_row + 1;
        p(l_html,'&nbsp;<a href="javascript:$.htmldbAjaxReport({'
          || 'pSeqId:' || pSeqId || ','
          || 'pName:''' || pName || ''','
          || 'pMaxRows:' || l_max_rows || ','
          || 'pStartRow:' || l_next_start
          || '})">'
        );
        p(l_html,'<img style="vertical-align:top" src="' || l_next_img || '" alt="" />');
        p(l_html,'</a>');
      END IF;
      p(l_html,'</div>');
    ELSE
      /* No data in report */
      p(l_html,'<span class="fielddatabold">No data found.</span>');
    END IF;
    p(l_html,'</td></tr></table></div></div>');
    --
    --
    /* store info to session */
    APEX_COLLECTION.UPDATE_MEMBER_ATTRIBUTE(
      p_collection_name => pName,
      p_seq             => pSeqId,
      p_attr_number     => 1,
      p_attr_value      => l_start_row
    );
    APEX_COLLECTION.UPDATE_MEMBER_ATTRIBUTE(
      p_collection_name => pName,
      p_seq             => pSeqId,
      p_attr_number     => 2,
      p_attr_value      => l_max_rows
    );
    APEX_COLLECTION.UPDATE_MEMBER_ATTRIBUTE(
      p_collection_name => pName,
      p_seq             => pSeqId,
      p_attr_number     => 3,
      p_attr_value      => l_sort_col
    );
    APEX_COLLECTION.UPDATE_MEMBER_ATTRIBUTE(
      p_collection_name => pName,
      p_seq             => pSeqId,
      p_attr_number     => 4,
      p_attr_value      => l_sort_ord
    );
    /*
    APEX_COLLECTION.UPDATE_MEMBER_ATTRIBUTE(
      p_collection_name => pName,
      p_seq             => pSeqId,
      p_attr_number     => 5,
      p_attr_value      => l_last_row
    );
    */
    --
    --
    /* print html */
    print_html(l_html);
  --
  EXCEPTION WHEN OTHERS THEN
    IF dbms_sql.is_open(l_cursor) THEN
      dbms_sql.close_cursor(l_cursor);
    END IF;
    htp.prn('<span class="fielddata">' || sqlerrm || '</span>');
  END;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE ajax_report_js
  AS
  /*
    Just for output JavaScript. JavaScript can be e.g. in page HTML header
    <script type="text/javascript">
      (function($){$.htmldbAjaxReport=function(o){
        $.ajax({
           url:"wwv_flow.show",
           dataType:"html",
           traditional:true,
           cache:false,
           type:"POST",
           data:{
            p_flow_id:$v("pFlowId"),
            p_flow_step_id:$v("pFlowStepId"),
            p_instance:$v("pInstance"),
            p_request:"APPLICATION_PROCESS=AJAX_REPORT_PPR",
            x01:o.pName,
            x02:o.pStartRow,
            x03:o.pMaxRows,
            x04:o.pSortCol,
            x05:o.pSeqId
           },
           dataFilter:function(d){
            return $(d).find("#htmldb_"+o.pName+o.pSeqId+"_catch")
           },
           success:function(r){
            $("#htmldb_"+o.pName+o.pSeqId+"_catch").replaceWith(r)
           }
          })
      }})(jQuery)
    </script>   
  */
  BEGIN
  --
    htp.prn('<script type="text/javascript">');
    --
    htp.prn('(function($){$.htmldbAjaxReport=function(o){');
    --
    htp.prn('
      $.ajax({
       url:"wwv_flow.show",
       dataType:"html",
       traditional:true,
       cache:false,
       type:"POST",
       data:{
        p_flow_id:$v("pFlowId"),
        p_flow_step_id:$v("pFlowStepId"),
        p_instance:$v("pInstance"),
        p_request:"APPLICATION_PROCESS=AJAX_REPORT_PPR",
        x01:o.pName,
        x02:o.pStartRow,
        x03:o.pMaxRows,
        x04:o.pSortCol,
        x05:o.pSeqId
       },
       dataFilter:function(d){return $(d).find("#htmldb_"+o.pName+o.pSeqId+"_catch")},
       success:function(r){$("#htmldb_"+o.pName+o.pSeqId+"_catch").replaceWith(r)}
      })
    ');
    --
    htp.prn('}})(jQuery)');
    --
    htp.p('</script>');
  --
  END;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE p(
    p_html IN OUT NOCOPY CLOB,
    p_append IN VARCHAR
  )
  AS
  BEGIN
    p_html := p_html || p_append;
  END;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE print_html(
    p_html IN CLOB
  )
  AS
    l_length      INTEGER;
    l_clob_len    INTEGER;
    l_offset      INTEGER;
    l_byte_len    INTEGER;
    l_temp        VARCHAR2(32767);
  BEGIN
  --
    l_length    := dbms_lob.getlength(p_html);
    l_clob_len  := l_length;
    l_byte_len  := 32700;
    l_offset    := 1;
    --
    IF l_length < l_byte_len THEN
      htp.prn(p_html);
    ELSE
    --
      WHILE l_offset < l_length AND l_byte_len > 0
      LOOP
        /* Get 32k cut */
        l_temp := dbms_lob.substr(p_html,l_byte_len,l_offset);
        --
        /* Print HTML */
        htp.prn(l_temp);
        --
        /* set the start position for the next cut */
        l_offset := l_offset + l_byte_len;
        --
        /* set the end position if less than 32k */
        l_clob_len := l_clob_len - l_byte_len;
        IF l_clob_len < l_byte_len THEN
          l_byte_len := l_clob_len;
        END IF;
      END LOOP;
    --
    END IF;
  --
  END;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
END;
/
